@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2009 Red Hat, Inc.
+# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,9 @@ class ConfigBase(object.Object):
     def set_value(self, section, name, value):
         pass
 
+    def unset_value(self, section, name):
+        pass
+
     def value_changed(self, section, name, value):
         self.__proxy.ValueChanged(section, name, value)
 
@@ -62,6 +65,9 @@ class ConfigProxy(interface.IConfig):
     def SetValue(self, section, name, value):
         return self.__config.set_value(section, name, value)
 
+    def UnsetValue(self, section, name):
+        return self.__config.unset_value(section, name)
+
     def Destroy(self):
         self.__config.destroy()
 
@@ -116,7 +122,8 @@ class Config(object.Object):
             self.__bus.remove_match(match_rule % self.__bus_name)
             self.__bus_name = None
 
-        self.__config = self.__bus.get_dbusconn().get_object(bus_name, IBUS_PATH_CONFIG)
+        _config = self.__bus.get_dbusconn().get_object(bus_name, IBUS_PATH_CONFIG)
+        self.__config = dbus.Interface(_config, dbus_interface="org.freedesktop.IBus.Config")
         self.__config.connect_to_signal("ValueChanged", self.__value_changed_cb)
 
         self.__bus_name = bus_name
@@ -141,3 +148,8 @@ class Config(object.Object):
     def set_list(self, section, name, value, signature):
         return self.set_value(section, name, dbus.Array(value, signature=signature))
 
+    def unset_value(self, section, name):
+        try:
+            return self.__config.UnsetValue(section, name)
+        except:
+            return
