@@ -30,15 +30,14 @@ import gobject
 import pango
 import ibus
 import keyboardshortcut
+import locale
 from os import path
 from xdg import BaseDirectory
 from gtk import gdk
 from enginecombobox import EngineComboBox
 from enginetreeview import EngineTreeView
 from engineabout import EngineAbout
-
-_  = lambda a : gettext.dgettext("ibus", a)
-N_ = lambda a : a
+from i18n import DOMAINNAME, _, N_, init as i18n_init
 
 (
     COLUMN_NAME,
@@ -68,12 +67,9 @@ class Setup(object):
 
     def __init__(self):
         super(Setup, self).__init__()
-        localedir = os.getenv("IBUS_LOCALEDIR")
-        gettext.bindtextdomain("ibus", localedir)
-        gettext.bind_textdomain_codeset("ibus", "UTF-8")
         gtk_builder_file = path.join(path.dirname(__file__), "./setup.ui")
         self.__builder = gtk.Builder()
-        self.__builder.set_translation_domain("ibus")
+        self.__builder.set_translation_domain(DOMAINNAME)
         self.__builder.add_from_file(gtk_builder_file);
         self.__bus = None
         self.__init_bus()
@@ -105,6 +101,28 @@ class Setup(object):
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("trigger"), "general/hotkey", "trigger", entry)
 
+        # enable (unconditional)
+        shortcuts = self.__config.get_value(
+                        "general/hotkey", "enable_unconditional",
+                        ibus.CONFIG_GENERAL_SHORTCUT_ENABLE_DEFAULT)
+        button = self.__builder.get_object("button_enable")
+        entry = self.__builder.get_object("entry_enable")
+        entry.set_text("; ".join(shortcuts))
+        entry.set_tooltip_text("\n".join(shortcuts))
+        button.connect("clicked", self.__shortcut_button_clicked_cb,
+                    N_("enable"), "general/hotkey", "enable_unconditional", entry)
+
+        # disable (unconditional)
+        shortcuts = self.__config.get_value(
+                        "general/hotkey", "disable_unconditional",
+                        ibus.CONFIG_GENERAL_SHORTCUT_DISABLE_DEFAULT)
+        button = self.__builder.get_object("button_disable")
+        entry = self.__builder.get_object("entry_disable")
+        entry.set_text("; ".join(shortcuts))
+        entry.set_tooltip_text("\n".join(shortcuts))
+        button.connect("clicked", self.__shortcut_button_clicked_cb,
+                    N_("disable"), "general/hotkey", "disable_unconditional", entry)
+
         # next engine
         shortcuts = self.__config.get_value(
                         "general/hotkey", "next_engine_in_menu",
@@ -459,5 +477,7 @@ class Setup(object):
         gtk.main()
 
 if __name__ == "__main__":
+    locale.setlocale(locale.LC_ALL, '')
+    i18n_init()
     setup = Setup()
     setup.run()
