@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
@@ -21,16 +22,15 @@
 #ifndef __IBUS_H_
 #define __IBUS_H_
 
+#define __IBUS_H_INSIDE__
+
 #include <ibusversion.h>
-#include <ibusmainloop.h>
 #include <ibusshare.h>
 #include <ibusobject.h>
 #include <ibusserializable.h>
 #include <ibusattribute.h>
 #include <ibusattrlist.h>
 #include <ibustext.h>
-#include <ibusconnection.h>
-#include <ibusserver.h>
 #include <ibusproxy.h>
 #include <ibusservice.h>
 #include <ibusfactory.h>
@@ -41,9 +41,6 @@
 #include <ibusbus.h>
 #include <ibuskeysyms.h>
 #include <ibuskeymap.h>
-#include <ibusmessage.h>
-#include <ibuspendingcall.h>
-#include <ibuserror.h>
 #include <ibusenumtypes.h>
 #include <ibushotkey.h>
 #include <ibusxml.h>
@@ -52,7 +49,9 @@
 #include <ibuscomponent.h>
 #include <ibusconfig.h>
 #include <ibusconfigservice.h>
-#include <ibusmainloop.h>
+#include <ibuspanelservice.h>
+
+#undef __IBUS_H_INSIDE__
 
 #endif
 
