@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
@@ -18,6 +19,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+#if !defined (__IBUS_H_INSIDE__) && !defined (IBUS_COMPILATION)
+#error "Only <ibus.h> can be included directly"
+#endif
+
 /**
  * SECTION: ibusengine
  * @short_description: Input method engine abstract.
@@ -59,6 +65,7 @@ G_BEGIN_DECLS
 
 typedef struct _IBusEngine IBusEngine;
 typedef struct _IBusEngineClass IBusEngineClass;
+typedef struct _IBusEnginePrivate IBusEnginePrivate;
 
 /**
  * IBusEngine:
@@ -70,7 +77,10 @@ typedef struct _IBusEngineClass IBusEngineClass;
  * IBusEngine properties.
  */
 struct _IBusEngine {
+    /*< private >*/
     IBusService parent;
+    IBusEnginePrivate *priv;
+
     /* instance members */
     /*< public >*/
     gboolean enabled;
@@ -82,9 +92,12 @@ struct _IBusEngine {
 };
 
 struct _IBusEngineClass {
+    /*< private >*/
     IBusServiceClass parent;
 
     /* class members */
+    /*< public >*/
+    /* signals */
     gboolean    (* process_key_event)
                                     (IBusEngine     *engine,
                                      guint           keyval,
@@ -123,10 +136,14 @@ struct _IBusEngineClass {
                                      guint           index,
                                      guint           button,
                                      guint           state);
+    void        (* set_surrounding_text)
+                                    (IBusEngine     *engine,
+                                     IBusText       *text,
+                                     guint           cursor_index);
 
     /*< private >*/
     /* padding */
-    gpointer pdummy[8];
+    gpointer pdummy[7];
 };
 
 GType        ibus_engine_get_type       (void);
@@ -135,14 +152,29 @@ GType        ibus_engine_get_type       (void);
  * ibus_engine_new:
  * @name: Name of the IBusObject.
  * @path: Path for IBusService.
- * @connection: An opened IBusConnection.
+ * @connection: An opened GDBusConnection.
  * @returns: A newly allocated IBusEngine.
  *
  * New an IBusEngine.
  */
-IBusEngine  *ibus_engine_new            (const gchar        *name,
-                                         const gchar        *path,
-                                         IBusConnection     *connection);
+IBusEngine  *ibus_engine_new            (const gchar        *engine_name,
+                                         const gchar        *object_path,
+                                         GDBusConnection    *connection);
+/**
+ * ibus_engine_new_type:
+ * @engine_type: GType of subclass of IBUS_TYPE_ENGINE
+ * @engine_name: Name of the IBusObject.
+ * @object_path: Path for IBusService.
+ * @connection: An opened GDBusConnection.
+ * @returns: A newly allocated IBusEngine.
+ *
+ * New an IBusEngine.
+ */
+IBusEngine  *ibus_engine_new_type       (GType               engine_type,
+                                         const gchar        *engine_name,
+                                         const gchar        *object_path,
+                                         GDBusConnection    *connection);
+
 
 /**
  * ibus_engine_commit_text:
@@ -365,6 +397,21 @@ void ibus_engine_delete_surrounding_text(IBusEngine         *engine,
                                          gint                offset,
                                          guint               nchars);
 
+/**
+ * ibus_engine_get_surrounding_text:
+ * @engine: An IBusEngine.
+ * @text: Location to store surrounding text.
+ * @cursor_pos: Cursor position in characters in @text.
+ *
+ * Get surrounding text.
+ *
+ * @see_also #IBusEngine::set-surrounding-text
+ */
+void ibus_engine_get_surrounding_text(IBusEngine         *engine,
+                                      IBusText          **text,
+                                      guint              *cursor_pos);
+
+
 /**
  * ibus_engine_get_name:
  * @engine: An IBusEngine.
