@@ -157,6 +157,36 @@ GType ibus_orientation_get_type (void) G_GNUC_CONST;
  * Returns the type of IBUS_ORIENTATION as GType.
  */
 #define IBUS_TYPE_ORIENTATION (ibus_orientation_get_type ())
+/**
+ * ibus_bus_name_flag_get_type:
+ * @returns: GType of IBUS_BUS_NAME_FLAG
+ *
+ * Returns the type of IBUS_BUS_NAME_FLAG as GType.
+ */
+GType ibus_bus_name_flag_get_type (void) G_GNUC_CONST;
+
+/**
+ * IBUS_TYPE_BUS_NAME_FLAG:
+ * @returns: GType of IBUS_BUS_NAME_FLAG
+ *
+ * Returns the type of IBUS_BUS_NAME_FLAG as GType.
+ */
+#define IBUS_TYPE_BUS_NAME_FLAG (ibus_bus_name_flag_get_type ())
+/**
+ * ibus_bus_request_name_reply_get_type:
+ * @returns: GType of IBUS_BUS_REQUEST_NAME_REPLY
+ *
+ * Returns the type of IBUS_BUS_REQUEST_NAME_REPLY as GType.
+ */
+GType ibus_bus_request_name_reply_get_type (void) G_GNUC_CONST;
+
+/**
+ * IBUS_TYPE_BUS_REQUEST_NAME_REPLY:
+ * @returns: GType of IBUS_BUS_REQUEST_NAME_REPLY
+ *
+ * Returns the type of IBUS_BUS_REQUEST_NAME_REPLY as GType.
+ */
+#define IBUS_TYPE_BUS_REQUEST_NAME_REPLY (ibus_bus_request_name_reply_get_type ())
 G_END_DECLS
 
 #endif /* __IBUS_ENUM_TYPES_H__ */
