@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,48 +28,19 @@
 #include "ibuskeysyms.h"
 #include "ibuskeymap.h"
 
+typedef guint KEYMAP[256][7];
 /* functions prototype */
-static void         ibus_keymap_class_init      (IBusKeymapClass        *klass);
-static void         ibus_keymap_init            (IBusKeymap             *keymap);
 static void         ibus_keymap_destroy         (IBusKeymap             *keymap);
 static gboolean     ibus_keymap_load            (const gchar            *name,
                                                  KEYMAP                  keymap);
-static IBusObjectClass *parent_class = NULL;
-static GHashTable      *keymaps = NULL;
+static GHashTable   *keymaps = NULL;
 
-GType
-ibus_keymap_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusKeymapClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_keymap_class_init,
-        NULL,               /* class finialize */
-        NULL,               /* class data */
-        sizeof (IBusKeymap),
-        0,
-        (GInstanceInitFunc) ibus_keymap_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_OBJECT,
-                                       "IBusKeymap",
-                                       &type_info,
-                                       0);
-    }
-
-    return type;
-}
+G_DEFINE_TYPE (IBusKeymap, ibus_keymap, IBUS_TYPE_OBJECT)
 
 static void
-ibus_keymap_class_init (IBusKeymapClass *klass)
+ibus_keymap_class_init (IBusKeymapClass *class)
 {
-    IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
-
-    parent_class = (IBusObjectClass *) g_type_class_peek_parent (klass);
+    IBusObjectClass *object_class = IBUS_OBJECT_CLASS (class);
 
     object_class->destroy = (IBusObjectDestroyFunc) ibus_keymap_destroy;
 }
@@ -90,7 +62,7 @@ ibus_keymap_destroy (IBusKeymap *keymap)
         g_free (keymap->name);
         keymap->name = NULL;
     }
-    IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *)keymap);
+    IBUS_OBJECT_CLASS (ibus_keymap_parent_class)->destroy ((IBusObject *)keymap);
 }
 
 #define SKIP_SPACE(p)   \
@@ -147,7 +119,7 @@ ibus_keymap_parse_line (gchar  *str,
     if (keycode == 0 && p1 == p2)
         return FALSE;
 
-    if (keycode < 0 || keycode > 255)
+    if ((int) keycode < 0 || keycode > 255)
         return FALSE;
 
     p1 = p2;
@@ -256,10 +228,17 @@ _keymap_destroy_cb (IBusKeymap *keymap,
                     gpointer    user_data)
 {
     g_hash_table_remove (keymaps, keymap->name);
+    g_object_unref (keymap);
 }
 
 IBusKeymap *
 ibus_keymap_new (const gchar *name)
+{
+    return ibus_keymap_get (name);
+}
+
+IBusKeymap *
+ibus_keymap_get (const gchar *name)
 {
     g_assert (name != NULL);
 
@@ -273,27 +252,28 @@ ibus_keymap_new (const gchar *name)
     }
 
     keymap = (IBusKeymap *) g_hash_table_lookup (keymaps, name);
-    if (keymap != NULL) {
-        g_object_ref (keymap);
-        return keymap;
+
+    if (keymap == NULL) {
+        keymap = g_object_new (IBUS_TYPE_KEYMAP, NULL);
+        g_object_ref_sink (keymap);
+
+        if (ibus_keymap_load (name, keymap->keymap)) {
+            ibus_keymap_fill (keymap->keymap);
+            keymap->name = g_strdup (name);
+            g_hash_table_insert (keymaps, g_strdup (keymap->name), keymap);
+
+            g_signal_connect (keymap, "destroy", G_CALLBACK (_keymap_destroy_cb), NULL);
+        }
+        else {
+            g_object_unref (keymap);
+            keymap = NULL;
+        }
     }
-
-    keymap = g_object_new (IBUS_TYPE_KEYMAP, NULL);
-    if (!ibus_keymap_load (name, keymap->keymap)) {
-        g_object_unref (keymap);
-        return NULL;
-    }
-
-    ibus_keymap_fill (keymap->keymap);
-    keymap->name = g_strdup (name);
-    g_hash_table_insert (keymaps, g_strdup (keymap->name), g_object_ref (keymap));
-
-    g_signal_connect (keymap, "destroy", G_CALLBACK (_keymap_destroy_cb), NULL);
-
+    if (keymap != NULL)
+        g_object_ref_sink (keymap);
     return keymap;
 }
 
-
 guint32
 ibus_keymap_lookup_keysym (IBusKeymap *keymap,
                            guint16     keycode,
