@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
@@ -19,16 +20,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "ibusshare.h"
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib-object.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <dbus/dbus.h>
-#include "ibusshare.h"
+#include <ibus.h>
 
 static gchar *_display = NULL;
 
@@ -38,9 +40,18 @@ ibus_get_local_machine_id (void)
     static gchar *machine_id = NULL;
 
     if (machine_id == NULL) {
-        gchar *id = dbus_get_local_machine_id ();
-        machine_id = g_strdup (id);
-        dbus_free (id);
+        GError *error = NULL;
+        if (!g_file_get_contents ("/var/lib/dbus/machine-id",
+                                  &machine_id,
+                                  NULL,
+                                  &error)) {
+            g_warning ("Unable to load /var/lib/dbus/machine-id: %s", error->message);
+            g_error_free (error);
+            machine_id = "machine-id";
+        }
+        else {
+            g_strstrip (machine_id);
+        }
     }
 
     return machine_id;
@@ -183,6 +194,27 @@ ibus_get_socket_path (void)
     return path;
 }
 
+gint
+ibus_get_timeout (void)
+{
+    /* 6000 ms is the default timeout on the ibus-daemon side (5 sec) plus 1. */
+    static const gint default_timeout = 6000;
+
+    static gint64 timeout = -2;
+    if (timeout == -2) {
+        const gchar *timeout_str = g_getenv ("IBUS_TIMEOUT");
+        if (timeout_str == NULL) {
+            timeout = default_timeout;
+        } else {
+            timeout = g_ascii_strtoll(timeout_str, NULL, 10);
+            if (timeout < -1 || timeout == 0 || timeout > G_MAXINT) {
+                timeout = default_timeout;
+            }
+        }
+    }
+    return timeout;
+}
+
 const gchar *
 ibus_get_address (void)
 {
@@ -197,7 +229,7 @@ ibus_get_address (void)
         address = NULL;
     }
 
-    /* get address from evn variable */
+    /* get address from env variable */
     address = g_strdup (g_getenv ("IBUS_ADDRESS"));
     if (address) {
         return address;
@@ -285,6 +317,12 @@ void
 ibus_init (void)
 {
     g_type_init ();
+    IBUS_TYPE_TEXT;
+    IBUS_TYPE_ATTRIBUTE;
+    IBUS_TYPE_ATTR_LIST;
+    IBUS_TYPE_LOOKUP_TABLE;
+    IBUS_TYPE_COMPONENT;
+    IBUS_TYPE_ENGINE_DESC;
 }
 
 static GMainLoop *main_loop = NULL;
@@ -307,3 +345,41 @@ ibus_quit (void)
         g_main_loop_quit (main_loop);
     }
 }
+
+static gboolean ibus_log_handler_is_verbose = FALSE;
+
+static void
+ibus_log_handler (const gchar    *log_domain,
+                  GLogLevelFlags  log_level,
+                  const gchar    *message,
+                  gpointer        user_data)
+{
+    // In the quiet mode (i.e. not verbose), we'll ignore DEBUG and
+    // WARNING messages.
+    if (!ibus_log_handler_is_verbose &&
+        ((log_level & G_LOG_LEVEL_DEBUG) ||
+         (log_level & G_LOG_LEVEL_WARNING))) {
+        return;
+    }
+    // Add timing info like "17:34:57.680038" (hour, min, sec, microsecond).
+    struct timeval time_val;
+    gettimeofday (&time_val, NULL);
+    struct tm local_time;
+    localtime_r (&time_val.tv_sec, &local_time);
+    char* new_message =
+        g_strdup_printf ("%02d:%02d:%02d.%6d: %s",
+                         local_time.tm_hour,
+                         local_time.tm_min,
+                         local_time.tm_sec,
+                         (int)time_val.tv_usec,
+                         message);
+    g_log_default_handler (log_domain, log_level, new_message, user_data);
+    g_free (new_message);
+}
+
+void
+ibus_set_log_handler (gboolean verbose)
+{
+    ibus_log_handler_is_verbose = verbose;
+    g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_MASK, ibus_log_handler, NULL);
+}
