@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,6 +19,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+#if !defined (__IBUS_H_INSIDE__) && !defined (IBUS_COMPILATION)
+#error "Only <ibus.h> can be included directly"
+#endif
+
 /**
  * SECTION: ibustypes
  * @short_description: Generic types for  IBus.
@@ -87,7 +93,7 @@ typedef enum
 
     IBUS_RELEASE_MASK  = 1 << 30,
 
-    IBUS_MODIFIER_MASK = 0x5c001fff
+    IBUS_MODIFIER_MASK = 0x5f001fff
 } IBusModifierType;
 
 /**
@@ -111,6 +117,18 @@ typedef enum {
     IBUS_CAP_SURROUNDING_TEXT   = 1 << 5,
 } IBusCapabilite;
 
+/**
+ * IBusPreeditFocusMode:
+ * @IBUS_ENGINE_PREEDIT_CLEAR: pre-edit text is cleared.
+ * @IBUS_ENGINE_PREEDIT_COMMIT: pre-edit text is committed.
+ *
+ * Pre-edit commit mode when the focus is lost.
+ */
+typedef enum {
+    IBUS_ENGINE_PREEDIT_CLEAR   = 0,
+    IBUS_ENGINE_PREEDIT_COMMIT  = 1,
+} IBusPreeditFocusMode;
+
 /**
  * IBusOrientation:
  * @IBUS_ORIENTATION_HORIZONTAL: Horizontal orientation.
@@ -125,6 +143,39 @@ typedef enum {
     IBUS_ORIENTATION_SYSTEM     = 2,
 } IBusOrientation;
 
+/**
+ * IBusBusNameFlag:
+ * @IBUS_BUS_NAME_FLAG_ALLOW_REPLACEMENT:
+ *    same as DBUS_NAME_FLAG_ALLOW_REPLACEMENT	
+ * @IBUS_BUS_NAME_FLAG_REPLACE_EXISTING:
+ *    same as DBUS_NAME_FLAG_REPLACE_EXISTING
+ * @IBUS_BUS_NAME_FLAG_DO_NOT_QUEUE:
+ *    same as DBUS_NAME_FLAG_DO_NOT_QUEUE 
+ */
+typedef enum {
+    IBUS_BUS_NAME_FLAG_ALLOW_REPLACEMENT   = (1 << 0),
+    IBUS_BUS_NAME_FLAG_REPLACE_EXISTING    = (1 << 1),
+    IBUS_BUS_NAME_FLAG_DO_NOT_QUEUE        = (1 << 2),
+} IBusBusNameFlag;
+
+/**
+ * IBusBusRequestNameReply:
+ * @IBUS_BUS_REQUEST_NAME_REPLY_PRIMARY_OWNER:
+ *    same as DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER
+ * @IBUS_BUS_REQUEST_NAME_REPLY_IN_QUEUE:
+ *    same as DBUS_REQUEST_NAME_REPLY_IN_QUEUE
+ * @IBUS_BUS_REQUEST_NAME_REPLY_EXISTS:
+ *    same as DBUS_REQUEST_NAME_REPLY_EXISTS
+ * @IBUS_BUS_REQUEST_NAME_REPLY_ALREADY_OWNER:
+ *    same as DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER
+ */
+typedef enum {
+    IBUS_BUS_REQUEST_NAME_REPLY_PRIMARY_OWNER   = 1,
+    IBUS_BUS_REQUEST_NAME_REPLY_IN_QUEUE        = 2,
+    IBUS_BUS_REQUEST_NAME_REPLY_EXISTS          = 3,
+    IBUS_BUS_REQUEST_NAME_REPLY_ALREADY_OWNER   = 4,
+} IBusBusRequestNameReply;
+
 /**
  * IBusRectangle:
  * @x: x coordinate.
