@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2009 Red Hat, Inc.
+# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -135,11 +135,19 @@ class ToggleToolButton(gtk.ToggleToolButton, PropItem):
             self.hide_all()
 
     def do_toggled(self):
+        # Do not send property-activate to engine in case the event is
+        # sent from engine.
+        do_emit = False
         if self.get_active():
+            if self._prop.state != ibus.PROP_STATE_CHECKED:
+                do_emit = True
             self._prop.state = ibus.PROP_STATE_CHECKED
         else:
+            if self._prop.state != ibus.PROP_STATE_UNCHECKED:
+                do_emit = True
             self._prop.state = ibus.PROP_STATE_UNCHECKED
-        self.emit("property-activate", self._prop.key, self._prop.state)
+        if do_emit:
+            self.emit("property-activate", self._prop.key, self._prop.state)
 
 class SeparatorToolItem(gtk.SeparatorToolItem, PropItem):
     __gtype_name__ = "IBusSeparatorToolItem"
